import asyncio
import base64
import json
import logging
from pathlib import Path
from types import SimpleNamespace
from typing import AsyncGenerator, Iterator

import httpx
import openai
import sqlitedict
from poemai_utils.basic_types_utils import linebreak, short_display
from poemai_utils.utils_config import get_config_by_key

_logger = logging.getLogger(__name__)


def current_unix_time():
    import datetime
    import time
    from types import SimpleNamespace

    unix_timestamp = int(time.time() * 1000)
    dt = datetime.datetime.fromtimestamp(unix_timestamp / 1000)
    # Convert datetime object to ISO date format
    iso_date = str(dt.isoformat())

    return SimpleNamespace(unix_timestamp=unix_timestamp, dt=dt, iso_date=iso_date)


class AskLlama:
    def __init__(
        self, model_path=None, n_ctx=2048, use_gpu=False, prompt_formatter=None
    ):
        try:
            from llama_cpp import Llama
        except ImportError:
            raise ImportError(
                "llama_cpp not installed - use pip install llama-cpp-python"
            )

        if model_path is None:
            model_path = get_config_by_key("LLAMA_MODEL_PATH")
            if model_path is None:
                raise ValueError("LLAMA_MODEL_PATH is not set")

        if prompt_formatter is None:
            self.prompt_formatter = lambda x: x
        else:
            self.prompt_formatter = prompt_formatter

        self.llama_model = Llama(
            model_path=model_path,
            n_gpu_layers=-1 if use_gpu else 0,  # Uncomment to use GPU acceleration
            n_ctx=n_ctx,
        )

        self.model = SimpleNamespace(name="llama")

    @staticmethod
    def prompt_formatter_mixtral():
        return lambda prompt: "[INST] " + prompt + " [/INST]"

    def count_tokens(self, text):
        """Returns the number of tokens in a text string."""
        encoding = self.llama_model.tokenize(text.encode("utf-8"))
        num_tokens = len(encoding)
        return num_tokens

    def ask_completion(
        self,
        prompt,
        temperature=0,
        max_tokens=600,
        stop=None,
        suffix=None,
        system_prompt=None,
        messages=None,
        json_mode=False,
    ):
        if system_prompt is not None:
            print(f"Warning: system_prompt is not supported for {self.llama_model}")
        if messages is not None:
            print(f"Warning: messages is not supported for {self.llama_model}")

        if stop is None:
            stop = []

        logits_processors = None

        if json_mode:
            from llama_cpp import LogitsProcessorList
            from lmformatenforcer.integrations.llamacpp import (
                build_llamacpp_logits_processor,
            )

            logits_processors = LogitsProcessorList(
                [
                    build_llamacpp_logits_processor(
                        self._tokenizer_data(), self._character_level_parser()
                    )
                ]
            )

        output = self.llama_model.create_completion(
            self.prompt_formatter(prompt),
            max_tokens=max_tokens,
            stop=stop,
            echo=False,
            stream=False,
            logits_processor=logits_processors,
        )

        return output["choices"][0]["text"]

    def ask(
        self,
        prompt,
        temperature=0,
        max_tokens=600,
        stop=None,
        suffix=None,
        system_prompt=None,
        messages=None,
        json_mode=False,
    ):
        answer = self.ask_completion(
            prompt,
            temperature,
            max_tokens,
            stop,
            suffix,
            system_prompt,
            messages,
            json_mode,
        )

        return answer

    async def ask_async(
        self,
        prompt,
        temperature=0,
        max_tokens=600,
        stop=None,
        system_prompt=None,
        messages=None,
        metadata=None,
    ) -> AsyncGenerator[dict, None]:
        if system_prompt is not None:
            print(f"Warning: system_prompt is not supported for {self.llama_model}")
        if messages is not None:
            print(f"Warning: messages is not supported for {self.llama_model}")

        if stop is None:
            stop = []

        generator = self.llama_model.create_completion(
            self.prompt_formatter(prompt),
            max_tokens=max_tokens,
            stop=stop,
            echo=False,
            stream=True,
        )

        # Adapt the synchronous generator for asynchronous iteration
        async for part in self._adapt_sync_gen_to_async(generator):
            _logger.debug(f"part: {part}")
            part_content = part.get("choices")[0]["text"]
            yield {"content": part_content}

        _logger.info(f"Finished async ask")

    async def _adapt_sync_gen_to_async(self, sync_gen):
        loop = asyncio.get_running_loop()

        while True:
            try:
                # Offload blocking call to executor
                next_item = await loop.run_in_executor(
                    None, next, sync_gen, StopIteration
                )
                if next_item is StopIteration:
                    break  # Properly terminate async iteration
                yield next_item
            except StopIteration:
                # This block may not be necessary if you use the default sentinel value as shown above
                break  # Ensure proper termination if StopIteration is raised

    def print_log(self, limit=10):
        for key in list(sorted(self.gpt_log.keys()))[-limit:]:
            entry = self.gpt_log[key]
            print(f"----------------------")
            print(f"model: {entry['model']} time: {entry['iso_date']} ")
            print(f"------ PROMPT  -------")
            print(f"{linebreak(entry['prompt'])}")
            print(f"------ ANSWER  -------")
            print(f"{linebreak(entry['answer'])}")
            print(f"**********************")

    def _tokenizer_data(self):
        from lmformatenforcer.integrations.llamacpp import (
            build_token_enforcer_tokenizer_data,
        )

        if self.tokenizer_data is None:
            self.tokenizer_data = build_token_enforcer_tokenizer_data(self.llama_model)
        return self.tokenizer_data

    def _character_level_parser(self):
        from lmformatenforcer import JsonSchemaParser

        if self.character_level_parser is None:
            self.character_level_parser = JsonSchemaParser(None)
        return self.character_level_parser



import json
import logging
from abc import ABC, abstractmethod

import sqlitedict
from poemai_utils.configuration_api import ConfigurationAPI

_logger = logging.getLogger(__name__)


class LLMAnswerCacheDao(ABC):
    @abstractmethod
    def store_llm_answer(self, prompt, answer, prompt_hash, answer_info_dict):
        raise NotImplementedError

    @abstractmethod
    def get_llm_answer(self, prompt_hash):
        raise NotImplementedError


class LLMAnswerCacheSqliteDict(LLMAnswerCacheDao):
    def __init__(self, config: ConfigurationAPI):
        self.cache = sqlitedict(
            config.LLM_ANSWER_CACHE_SQLITE_DICT_PATH, autocommit=True
        )

    def store_llm_answer(self, prompt_hash, answer_info_dict):
        self.cache[prompt_hash] = answer_info_dict

        return prompt_hash

    def get_llm_answer(self, prompt_hash):
        return self.cache[prompt_hash]


class LLMAnswerCacheDaoDict(LLMAnswerCacheDao):
    def __init__(self):
        self.cache = {}

    def store_llm_answer(self, prompt_hash, answer_info_dict):
        self.cache[prompt_hash] = answer_info_dict

        return prompt_hash

    def get_llm_answer(self, prompt_hash):
        return self.cache.get(prompt_hash, None)



from enum import Enum

from poemai_utils.enum_utils import add_enum_attrs, add_enum_repr


class SentenceTransformerEmbeddingModel(Enum):
    LABSE = "sentence-transformers/LaBSE"
    DISTILUSE = "distiluse-base-multilingual-cased-v1"
    BI_ELECTRA_GERMAN = "svalabs/bi-electra-ms-marco-german-uncased"
    DISTILBERT = "msmarco-distilbert-base-tas-b"


add_enum_attrs(
    {
        SentenceTransformerEmbeddingModel.LABSE: {
            "use_cosine_similarity": False,
            "embeddings_dimensions": 768,
        },
        SentenceTransformerEmbeddingModel.DISTILUSE: {
            "use_cosine_similarity": False,
            "embeddings_dimensions": 768,
        },
        SentenceTransformerEmbeddingModel.BI_ELECTRA_GERMAN: {  # best german embeddings found so far
            "use_cosine_similarity": True,
            "embeddings_dimensions": 768,
        },
        SentenceTransformerEmbeddingModel.DISTILBERT: {
            "use_cosine_similarity": False,
            "embeddings_dimensions": 768,
        },
    }
)
add_enum_repr(SentenceTransformerEmbeddingModel)



from hashlib import sha256

import numpy as np
from poemai_utils.embeddings.embedder_base import EmbedderBase
from poemai_utils.embeddings.embedding_cache import EmbeddingCache


class EmbeddingStore:
    def __init__(self, embedding_cache=None, embedder=None):
        self.embedder = embedder
        if self.embedder is None:
            raise ValueError("embedder must be provided")
        if not isinstance(self.embedder, EmbedderBase):
            raise ValueError("embedder must be an EmbedderBase object")

        self.texts = []
        self.embedding_matrix = None
        self.embedding_cache = embedding_cache
        if self.embedding_cache is not None:
            if not isinstance(self.embedding_cache, EmbeddingCache):
                raise ValueError("embedding_cache must be an EmbeddingCache object")

    def _store_embedding(self, embedding):
        if self.embedding_matrix is None:
            self.embedding_matrix = np.expand_dims(embedding, 0)
        else:
            self.embedding_matrix = np.concatenate(
                [self.embedding_matrix, [embedding]], 0
            )

    def add_text(self, text, is_query: bool = False):
        self.texts.append(text)
        self._store_embedding(self._create_embedding(text, is_query=is_query))
        return len(self.texts) - 1

    def add_embedding(self, text, embedding):
        self.texts.append(text)
        self._store_embedding(embedding)
        return len(self.texts) - 1

    def query_by_text(self, query, k=5):
        if self.embedding_matrix is None:
            return []
        query_embedding = self._create_embedding(query, is_query=True)
        embedding_matrix = self._scaled_embedding_matrix()

        scores = embedding_matrix.dot(query_embedding)
        idxs = np.argsort(scores)[-k:][::-1]

        return [(self.texts[i], scores[i], i) for i in idxs]

    def _scaled_embedding_matrix(self):
        if self.embedding_matrix is None:
            return None
        if self.embedder.use_cosine_similarity:
            return self.embedding_matrix / np.linalg.norm(
                self.embedding_matrix, ord=2, axis=1, keepdims=True
            )
        else:
            return self.embedding_matrix

    def query_by_embedding(self, query_embedding, k=5):
        if self.embedding_matrix is None:
            return []

        embedding_matrix = self._scaled_embedding_matrix()
        scores = embedding_matrix.dot(query_embedding)

        scores = self.embedding_matrix.dot(query_embedding)
        idxs = np.argsort(scores)[-k:][::-1]

        return [(self.texts[i], scores[i], i) for i in idxs]

    def _create_embedding(self, text, is_query):
        if self.embedding_cache is not None:
            embedding = self.embedding_cache.get(text, self.embedder.model_name)
            if embedding is not None:
                return embedding

        embedding = self.embedder.calc_embedding(text, is_query=is_query)

        if self.embedding_cache is not None:
            self.embedding_cache.put(text, embedding, self.embedder.model_name)

        return embedding

    def similarity(self, index_1, index_2):
        if self.embedding_matrix is None:
            return None

        embedding_matrix = self._scaled_embedding_matrix()

        score = embedding_matrix[index_1].dot(embedding_matrix[index_2])
        return score



import copy
import json
import logging
import re
import threading

from poemai_utils.aws.dynamodb import DynamoDB, VersionMismatchException
from sqlitedict import SqliteDict

_logger = logging.getLogger(__name__)


class DynamoDBEmulator:
    def __init__(self, sqlite_filename):
        self.data_table = SqliteDict(sqlite_filename, tablename="data")
        self.index_table = SqliteDict(sqlite_filename, tablename="index")
        self.lock = threading.Lock()

    def _get_composite_key(self, table_name, pk, sk):
        return f"{table_name}___##___{pk}___##___{sk}"

    def _get_pk_sk_from_composite_key(self, composite_key):
        key_components = composite_key.split("___##___")[1:3]
        return key_components[0], key_components[1]

    def _get_index_key(self, table_name, pk):
        return f"{table_name}#{pk}"

    def get_all_items(self):
        for k, v in self.data_table.items():
            pk, sk = self._get_pk_sk_from_composite_key(k)

            yield {"pk": pk, "sk": sk, **v}

    def store_item(self, table_name, item):
        with self.lock:
            pk = item["pk"]
            sk = item.get("sk", "")

            composite_key = self._get_composite_key(table_name, pk, sk)

            # Store the item
            self.data_table[composite_key] = item
            self.data_table.commit()

            index_key = self._get_index_key(table_name, pk)
            index_list = set(self.index_table.get(index_key, []))

            index_list.add(composite_key)

            self.index_table[index_key] = index_list
            self.index_table.commit()

    def update_versioned_item_by_pk_sk(
        self,
        table_name,
        pk,
        sk,
        attribute_updates,
        expected_version,
        version_attribute_name="version",
    ):
        with self.lock:
            composite_key = self._get_composite_key(table_name, pk, sk)
            item = self.data_table.get(composite_key)

            # If the item does not exist, we cannot update it
            if item is None:
                raise KeyError(f"Item with pk:{pk} and sk:{sk} does not exist.")

            # Check for version mismatch
            if item.get(version_attribute_name, 0) != expected_version:
                raise VersionMismatchException(
                    f"Version mismatch for item {pk}:{sk}. "
                    f"Current version: {item.get(version_attribute_name, 0)}, "
                    f"expected: {expected_version}."
                )

            # Update the item's attributes
            for attr, value in attribute_updates.items():
                item[attr] = value

            # Update the version
            item[version_attribute_name] = expected_version + 1

            # Store the updated item
            self.data_table[composite_key] = item
            self.data_table.commit()

    def get_item_by_pk_sk(self, table_name, pk, sk):
        composite_key = self._get_composite_key(table_name, pk, sk)

        retval = self.data_table.get(composite_key, None)
        if retval:
            retval["pk"] = pk
            retval["sk"] = sk
        return retval

    def get_item_by_pk(self, table_name, pk):
        composite_key = self._get_composite_key(table_name, pk, "")
        retval = self.data_table.get(composite_key, None)
        if retval:
            retval["pk"] = pk
        return retval

    def get_paginated_items_by_pk(self, table_name, pk, limit=None):
        results = []
        index_key = self._get_index_key(table_name, pk)
        composite_keys = set(self.index_table.get(index_key, []))
        for composite_key in sorted(composite_keys):
            item = self.data_table.get(composite_key, None)
            if item:
                pk, sk = self._get_pk_sk_from_composite_key(composite_key)
                new_item = copy.deepcopy(item)
                new_item["pk"] = pk
                new_item["sk"] = sk
                results.append(new_item)

        return results

    def delete_item_by_pk_sk(self, table_name, pk, sk):
        composite_key = self._get_composite_key(table_name, pk, sk)

        # Delete the item
        del self.data_table[composite_key]
        self.data_table.commit()

        # Delete the index
        index_key = self._get_index_key(table_name, pk)
        index_list = self.index_table.get(index_key, [])
        index_list.remove(composite_key)
        self.index_table[index_key] = index_list
        self.index_table.commit()

    def scan_for_items_by_pk_sk(self, table_name, pk_contains, sk_contains):
        raise NotImplementedError("scan_for_items_by_pk_sk not implemented")

    def query(
        self,
        TableName,
        KeyConditionExpression,
        ExpressionAttributeValues,
        ProjectionExpression=None,
    ):
        """A very simplistic implementation for DynamoDB query operation. It only supports
        equality and begins_with operators in the KeyConditionExpression. It does not
        support any other operations like filter expressions, etc. It also does not
        support any index operations. It is only meant to be used for testing purposes.
        """

        # Helper function to evaluate conditions
        def evaluate_condition(item, key, operator, value):
            if operator == "=" and item.get(key) == value:
                return True
            elif operator == "begins_with" and item.get(key, "").startswith(value):
                return True
            return False

        # Parse the KeyConditionExpression
        conditions = KeyConditionExpression.split(" and ")
        parsed_conditions = []
        for condition in conditions:
            if "begins_with" in condition:
                key, value = re.match(
                    r"begins_with\((\w+), :(\w+)\)", condition
                ).groups()
                operator = "begins_with"
            else:
                key, value = re.match(r"(\w+) = :(\w+)", condition).groups()
                operator = "="
            parsed_conditions.append((key, operator, value))

        # Replace placeholders with actual values
        for i, (key, operator, placeholder) in enumerate(parsed_conditions):
            value_dict = ExpressionAttributeValues.get(f":{placeholder}")
            if value_dict:
                value = next(
                    iter(value_dict.values())
                )  # Get the value from dict e.g., {"S": "some_value"}
                parsed_conditions[i] = (key, operator, value)

        # Perform full table scan and filter results
        results = []
        for k, v in self.data_table.items():
            # Extract table name, pk, and sk from the composite key
            key_parts = k.split("___##___")
            if key_parts[0] != TableName:
                continue  # Skip items that do not belong to the specified table

            pk, sk = key_parts[1], key_parts[2]
            item = {"pk": pk, "sk": sk, **v}
            # Check all conditions
            if all(
                evaluate_condition(item, key, operator, value)
                for key, operator, value in parsed_conditions
            ):
                # If projection is specified, filter the keys
                if ProjectionExpression:
                    projected_item = {
                        k: v
                        for k, v in item.items()
                        if k in ProjectionExpression.split(",")
                    }
                    results.append(projected_item)
                else:
                    results.append(item)

        results = {"Items": [DynamoDB.dict_to_item(item) for item in results]}

        _logger.info(f"Query results: {json.dumps(results, indent=2)}")

        return results

    def item_exists(self, table_name, pk, sk):
        composite_key = self._get_composite_key(table_name, pk, sk)
        return composite_key in self.data_table



import numpy as np
from poemai_utils.embeddings.embedding_store import EmbeddingStore


class SelfSimilarityAnalyzer:
    def __init__(self, embedder, embedding_cache=None):
        if embedder is None:
            raise ValueError("embedder must be provided")
        self.embedding_store = EmbeddingStore(embedding_cache, embedder=embedder)
        self.use_cosine_similarity = embedder.use_cosine_similarity
        self.metadata = {}

    def add_text(self, text, metadata, is_query=False):
        if text is None:
            return
        text_id = self.embedding_store.add_text(text, is_query=is_query)
        self.metadata[text_id] = metadata

    def add_embedding(self, text, embedding, metadata):
        text_id = self.embedding_store.add_embedding(text, embedding)
        self.metadata[text_id] = metadata

    def query(self, query, k=5):
        top_k = self.embedding_store.query_by_text(query, k)
        return [
            (int(text_id), float(score), self.metadata.get(text_id))
            for _, score, text_id in top_k
        ]

    @staticmethod
    def normalize_rows(x):
        return x / np.linalg.norm(x, ord=2, axis=1, keepdims=True)

    def similarity_matrix(self):
        m1 = self.embedding_store.embedding_matrix
        if self.use_cosine_similarity:
            m1 = self.normalize_rows(m1)

        return np.matmul(
            m1,
            m1.T,
        )

    def find_top_k_similar_pairs(self, k=5):
        similarity_matrix = self.similarity_matrix()

        similarity_matrix = np.tril(similarity_matrix)
        np.fill_diagonal(similarity_matrix, 0)

        # find the top k most similar sections
        top_k_idxs = np.argsort(similarity_matrix, axis=None)[-k:]
        rows, cols = np.unravel_index(top_k_idxs, similarity_matrix.shape)
        values = similarity_matrix[rows, cols]

        retval = []
        for row, col, value in zip(rows, cols, values):
            row_metadata = self.metadata.get(row)
            col_metadata = self.metadata.get(col)

            retval.append(
                (
                    row_metadata,
                    col_metadata,
                    value,
                )
            )

        return retval

    def metadata(self, text_id):
        return self.metadata.get(text_id)



from poemai_utils.embeddings.embedder_base import EmbedderBase
from poemai_utils.embeddings.openai_embedder import OpenAIEmbedder
from poemai_utils.embeddings.sentence_transformer_embedder import (
    SentenceTransformerEmbedder,
    SentenceTransformerEmbeddingModel,
)
from poemai_utils.openai.openai_model import API_TYPE, OPENAI_MODEL


def make_embedder(model_id: str, **kwargs) -> EmbedderBase:
    try:
        model_id_enum = SentenceTransformerEmbeddingModel(model_id)
        return SentenceTransformerEmbedder(model_id_enum, **kwargs)
    except ValueError:
        pass

    openai_model_id_enum = None
    try:
        openai_model_id_enum = OPENAI_MODEL.by_model_key(model_id)
    except ValueError:
        pass
    if openai_model_id_enum is not None:
        if API_TYPE.EMBEDDINGS in openai_model_id_enum.api_types:
            return OpenAIEmbedder(openai_model_id_enum, **kwargs)
        else:
            raise ValueError(f"Model {model_id} does not support embeddings")
    else:
        raise ValueError(f"Unknown model_id: {model_id}")



from enum import Enum

from poemai_utils.enum_utils import add_enum_attrs, add_enum_repr, add_enum_repr_attr


class OPENAI_MODEL(str, Enum):
    GPT_4_TURBO = "gpt_4_turbo"
    GPT_4_TURBO_PREVIEW = "gpt-4-turbo-preview"
    GPT_4_0125_PREVIEW = "gpt-4-0125-preview"
    GPT_4_TURBO_1106_PREVIEW = "gpt_4_turbo_1106_preview"
    GPT_4_VISION_PREVIEW = "gpt-4-vision-preview"
    GPT_3_5_TURBO = "gpt_3_5_turbo"
    GPT_3_5_TURBO_1106 = "gpt-3.5-turbo-1106"
    GPT_3_5_TURBO_0613 = "gpt_3_5_turbo_0613"
    GPT_3_5_TURBO_16k = "gpt_3_5_turbo_16k"
    GPT_3_5_TURBO_0125 = "gpt-3.5-turbo-0125"
    ADA_002_EMBEDDING = "text-embedding-ada-002"
    TEXT_EMBEDDING_3_LARGE = "text-embedding-3-large"
    TEXT_EMBEDDING_3_SMALL = "text-embedding_3-small"

    @classmethod
    def by_model_key(cls, model_key):
        if model_key.startswith("openai."):
            model_key = model_key[7:]
        for model in cls:
            if model.model_key == model_key:
                return model
        raise ValueError(f"Unknown model_key: {model_key}")


add_enum_repr_attr(OPENAI_MODEL)


class API_TYPE(Enum):
    COMPLETIONS = "completions"
    CHAT_COMPLETIONS = "chat_completions"
    EMBEDDINGS = "embeddings"
    MODERATIONS = "moderations"


add_enum_repr(API_TYPE)

add_enum_attrs(
    {
        OPENAI_MODEL.GPT_4_TURBO: {
            "model_key": "gpt-4-0125-preview",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_4_TURBO_PREVIEW: {
            "model_key": "gpt-4-turbo-preview",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_4_0125_PREVIEW: {
            "model_key": "gpt-4-0125-preview",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_4_TURBO_1106_PREVIEW: {
            "model_key": "gpt-4-1106-preview",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_4_VISION_PREVIEW: {
            "model_key": "gpt-4-vision-preview",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": True,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_3_5_TURBO: {
            "model_key": "gpt-3.5-turbo",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": False,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_3_5_TURBO_1106: {
            "model_key": "gpt-3.5-turbo-1106",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": False,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_3_5_TURBO_16k: {
            "model_key": "gpt-3.5-turbo-16k",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_3_5_TURBO_0125: {
            "model_key": "gpt-3.5-turbo-0125",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.GPT_3_5_TURBO_0613: {
            "model_key": "gpt-3.5-turbo-0613",
            "api_types": [API_TYPE.CHAT_COMPLETIONS],
            "expensive": True,
            "supports_vision": False,
            "embeddings_dimensions": None,
        },
        OPENAI_MODEL.ADA_002_EMBEDDING: {
            "model_key": "text-embedding-ada-002",
            "api_types": [API_TYPE.EMBEDDINGS],
            "expensive": False,
            "supports_vision": False,
            "embeddings_dimensions": 1536,
        },
        OPENAI_MODEL.TEXT_EMBEDDING_3_LARGE: {
            "model_key": "text-embedding-3-large",
            "api_types": [API_TYPE.EMBEDDINGS],
            "expensive": False,
            "supports_vision": False,
            "embeddings_dimensions": 3072,
        },
        OPENAI_MODEL.TEXT_EMBEDDING_3_SMALL: {
            "model_key": "text-embedding-3-small",
            "api_types": [API_TYPE.EMBEDDINGS],
            "expensive": False,
            "supports_vision": False,
            "embeddings_dimensions": 1536,
        },
    }
)



import logging

import numpy as np
from poemai_utils.embeddings.embedder_base import EmbedderBase
from poemai_utils.openai.openai_model import API_TYPE, OPENAI_MODEL

_logger = logging.getLogger(__name__)


class OpenAIEmbedder(EmbedderBase):
    def __init__(self, model_name="text-embedding-ada-002", openai_api_key=None):
        _logger.info(f"Start initializing OpenAIEmbedder with model {model_name}")

        try:
            from openai import OpenAI
        except ImportError:
            raise ImportError(
                "You must install openai to use this function. Try: pip install openai"
            )

        super().__init__()

        self.model_name = model_name

        try:
            openai_model_id_enum = OPENAI_MODEL(model_name)
        except ValueError:
            raise ValueError(f"Unknown model_id: {model_name}")

        if API_TYPE.EMBEDDINGS not in openai_model_id_enum.api_types:

            raise ValueError(f"Model {model_name} does not support embeddings")

        self.openai_model = openai_model_id_enum
        if openai_api_key is not None:
            self.client = OpenAI(api_key=openai_api_key)
        else:
            self.client = OpenAI()

        _logger.info(f"Initialized OpenAIEmbedder with model {model_name}")

    def calc_embedding(self, text, is_query: bool = False):
        response = self.client.embeddings.create(input=text, model=self.model_name)
        embedding = response.data[0].embedding
        embedding = np.array(embedding, dtype=np.float32)
        return embedding

    def embedding_dim(self):
        return self.openai_model.embeddings_dimensions
